@@ -51,6 +51,9 @@ CBotDefClass::CBotDefClass()
 ////////////////////////////////////////////////////////////////////////////////
 CBotDefClass::~CBotDefClass()
 {
+    delete m_parameters;
+    delete m_exprRetVar;
+    delete m_expr;
     delete m_var;
 }
 
@@ -131,7 +134,7 @@ CBotInstr* CBotDefClass::Compile(CBotToken* &p, CBotCStack* pStack, CBotClass* p
         {
             // the constructor is there?
 //          std::string  noname;
-            CBotTypResult r = pClass->CompileMethode(pClass->GetName(), var, ppVars, pStk, inst->m_nMethodeIdent);
+            CBotTypResult r = pClass->CompileMethode(&token, var, ppVars, pStk, inst->m_nMethodeIdent);
             delete pStk->TokenStack();                          // releases the supplement stack
             int typ = r.GetType();
 
@@ -246,9 +249,9 @@ bool CBotDefClass::Execute(CBotStack* &pj)
 
     if (m_exprRetVar != nullptr) // Class c().method();
     {
-        if (pile->IfStep()) return false;
         if (pile->GetState() == 4)
         {
+            if (pile->IfStep()) return false;
             CBotStack* pile3 = pile->AddStack();
             if (!m_exprRetVar->Execute(pile3)) return false;
             pile3->SetVar(nullptr);
@@ -369,11 +372,7 @@ bool CBotDefClass::Execute(CBotStack* &pj)
             ppVars[i] = nullptr;
 
             // creates a variable for the result
-            CBotVar*    pResult = nullptr;     // constructor still void
-
-            if ( !pClass->ExecuteMethode(m_nMethodeIdent, pClass->GetName(),
-                                         pThis, ppVars,
-                                         pResult, pile2, GetToken())) return false; // interrupt
+            if ( !pClass->ExecuteMethode(m_nMethodeIdent, pThis, ppVars, CBotTypResult(CBotTypVoid), pile2, GetToken())) return false; // interrupt
 
             pThis->SetInit(CBotVar::InitType::DEF);
             pThis->ConstructorSet();        // indicates that the constructor has been called
@@ -483,9 +482,7 @@ void CBotDefClass::RestoreState(CBotStack* &pj, bool bMain)
             ppVars[i] = nullptr;
 
             // creates a variable for the result
-//            CBotVar*    pResult = nullptr;     // constructor still void
-
-            pClass->RestoreMethode(m_nMethodeIdent, pClass->GetName(), pThis, ppVars, pile2);
+            pClass->RestoreMethode(m_nMethodeIdent, pt, pThis, ppVars, pile2);
             return;
         }
     }
