@@ -22,6 +22,7 @@
 
 #include "CBot/CBotStack.h"
 
+#include "CBot/CBotInstr/CBotInstr.h"
 #include "CBot/CBotVar/CBotVarArray.h"
 #include "CBot/CBotVar/CBotVarPointer.h"
 #include "CBot/CBotVar/CBotVarClass.h"
@@ -61,10 +62,17 @@ CBotVar::CBotVar( )
     m_mPrivate = ProtectionLevel::Public;
 }
 
+CBotVar::CBotVar(const CBotToken &name) : CBotVar()
+{
+    m_token = new CBotToken(name);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 CBotVar::~CBotVar( )
 {
     delete  m_token;
+    delete  m_InitExpr;
+    delete  m_LimExpr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -698,7 +706,16 @@ void CBotVar::Dec()
 ////////////////////////////////////////////////////////////////////////////////
 void CBotVar::Copy(CBotVar* pSrc, bool bName)
 {
-    assert(0);
+    if (bName) *m_token = *pSrc->m_token;
+    m_type = pSrc->m_type;
+    m_binit = pSrc->m_binit;
+//-    m_bStatic    = pSrc->m_bStatic;
+    m_next = nullptr;
+    m_pMyThis = nullptr;//p->m_pMyThis;
+    m_pUserPtr = pSrc->m_pUserPtr;
+
+    // keeps indentificator the same (by default)
+    if (m_ident == 0) m_ident = pSrc->m_ident;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -727,4 +744,39 @@ CBotClass* CBotVar::GetClass()
     return nullptr;
 }
 
+CBotVar::operator int()
+{
+    return GetValInt();
+}
+
+CBotVar::operator float()
+{
+    return GetValFloat();
+}
+
+CBotVar::operator std::string()
+{
+    return GetValString();
+}
+
+void CBotVar::operator=(const CBotVar &var)
+{
+    SetVal(const_cast<CBotVar*>(&var));
+}
+
+void CBotVar::operator=(int x)
+{
+    SetValInt(x);
+}
+
+void CBotVar::operator=(float x)
+{
+    SetValFloat(x);
+}
+
+void CBotVar::operator=(const std::string &x)
+{
+    SetValString(x);
+}
+
 } // namespace CBot
