@@ -50,6 +50,11 @@ public:
      */
     CBotVar();
 
+    /**
+     * \brief Constructor. Do not call directly, use CBotVar::Create()
+     */
+    CBotVar(const CBotToken& name);
+
     /**
      * \brief Destructor. Do not call directly, use CBotVar::Destroy()
      */
@@ -439,6 +444,14 @@ public:
      */
     //@{
 
+    operator int();
+    operator float();
+    operator std::string();
+    void operator=(const CBotVar& var);
+    void operator=(int x);
+    void operator=(float x);
+    void operator=(const std::string &x);
+
     /**
      * \brief Set the value
      * \param var Another variable to copy value from
