@@ -95,6 +95,10 @@ enum EventType
     //! Event sent on user quit request
     EVENT_QUIT              = 20,
     EVENT_UPDINTERFACE      = 21,
+    //! Event sent on resolution change
+    EVENT_RESOLUTION_CHANGED = 22,
+    //! Event sent when textures have to be reloaded
+    EVENT_RELOAD_TEXTURES   = 23,
     EVENT_WIN               = 30,
     EVENT_LOST              = 31,
 
@@ -145,7 +149,7 @@ enum EventType
     EVENT_WINDOW4           = 84,   //!< CDisplayInfo
     EVENT_WINDOW5           = 85,   //!< all menu windows
     EVENT_WINDOW6           = 86,   //!< code battle interface
-    EVENT_WINDOW7           = 87,   //!< (unused)
+    EVENT_WINDOW7           = 87,   //!< debug interface
     EVENT_WINDOW8           = 88,   //!< (unused)
     EVENT_WINDOW9           = 89,   //!< CMainDialog and CStudio file selector
 
@@ -168,7 +172,7 @@ enum EventType
     EVENT_LABEL16           = 106,
     EVENT_LABEL17           = 107,
     EVENT_LABEL18           = 108,
-    EVENT_LABEL19           = 109,
+    EVENT_LABEL19           = 109, // cursor position overlay
 
     EVENT_LIST0             = 110,
     EVENT_LIST1             = 111,
@@ -183,6 +187,11 @@ enum EventType
 
     EVENT_LOADING           = 120,
 
+    EVENT_LABEL_CODE_BATTLE = 121,
+
+    EVENT_SCOREBOARD        = 130,
+    EVENT_SCOREBOARD_MAX    = 169,
+
     EVENT_TOOLTIP           = 200,
 
     EVENT_DIALOG_OK         = 300,
@@ -233,6 +242,7 @@ enum EventType
     EVENT_INTERFACE_LIGHT   = 457,
     EVENT_INTERFACE_PARTI   = 458,
     EVENT_INTERFACE_CLIP    = 459,
+    EVENT_INTERFACE_PAUSE_BLUR = 460,
     EVENT_INTERFACE_RAIN    = 462,
     EVENT_INTERFACE_GLINT   = 463,
     EVENT_INTERFACE_TOOLTIP = 464,
@@ -280,24 +290,30 @@ enum EventType
 
     EVENT_INTERFACE_JOYSTICK= 560,
     EVENT_INTERFACE_SOLUCE  = 561,
-    EVENT_INTERFACE_JOYSTICK_X = 562,
-    EVENT_INTERFACE_JOYSTICK_Y = 563,
-    EVENT_INTERFACE_JOYSTICK_Z = 564,
-    EVENT_INTERFACE_JOYSTICK_X_INVERT = 565,
-    EVENT_INTERFACE_JOYSTICK_Y_INVERT = 566,
-    EVENT_INTERFACE_JOYSTICK_Z_INVERT = 567,
-    EVENT_INTERFACE_JOYSTICK_DEADZONE = 568,
+    EVENT_INTERFACE_JOYSTICK_DEADZONE = 562,
+    EVENT_INTERFACE_JOYSTICK_X = 563,
+    EVENT_INTERFACE_JOYSTICK_Y = 564,
+    EVENT_INTERFACE_JOYSTICK_Z = 565,
+    EVENT_INTERFACE_JOYSTICK_CAM_X = 566,
+    EVENT_INTERFACE_JOYSTICK_CAM_Y = 567,
+    EVENT_INTERFACE_JOYSTICK_CAM_Z = 568,
+    EVENT_INTERFACE_JOYSTICK_X_INVERT = 569,
+    EVENT_INTERFACE_JOYSTICK_Y_INVERT = 570,
+    EVENT_INTERFACE_JOYSTICK_Z_INVERT = 571,
+    EVENT_INTERFACE_JOYSTICK_CAM_X_INVERT = 572,
+    EVENT_INTERFACE_JOYSTICK_CAM_Y_INVERT = 573,
+    EVENT_INTERFACE_JOYSTICK_CAM_Z_INVERT = 574,
 
-    EVENT_INTERFACE_GLINTl  = 570,
-    EVENT_INTERFACE_GLINTr  = 571,
-    EVENT_INTERFACE_GLINTu  = 572,
-    EVENT_INTERFACE_GLINTb  = 573,
+    EVENT_INTERFACE_GLINTl  = 590,
+    EVENT_INTERFACE_GLINTr  = 591,
+    EVENT_INTERFACE_GLINTu  = 592,
+    EVENT_INTERFACE_GLINTb  = 593,
 
-    EVENT_INTERFACE_NEDIT   = 580,
-    EVENT_INTERFACE_NLIST   = 581,
-    EVENT_INTERFACE_NOK     = 582,
-    EVENT_INTERFACE_NDELETE = 584,
-    EVENT_INTERFACE_NLABEL  = 585,
+    EVENT_INTERFACE_NEDIT   = 595,
+    EVENT_INTERFACE_NLIST   = 596,
+    EVENT_INTERFACE_NOK     = 597,
+    EVENT_INTERFACE_NDELETE = 598,
+    EVENT_INTERFACE_NLABEL  = 599,
 
     EVENT_INTERFACE_IOWRITE = 600,
     EVENT_INTERFACE_IOREAD  = 601,
@@ -376,6 +392,31 @@ enum EventType
     EVENT_CMD               = 800,
     EVENT_SPEED             = 801,
 
+    EVENT_DBG_STATS         = 850,
+    EVENT_DBG_SPAWN_OBJ     = 851,
+    EVENT_DBG_TELEPORT      = 852,
+    EVENT_DBG_LIGHTNING     = 853,
+    EVENT_DBG_RESOURCES     = 854,
+    EVENT_DBG_GOTO          = 855,
+    EVENT_DBG_CRASHSPHERES  = 856,
+    EVENT_DBG_LIGHTS        = 857,
+    EVENT_DBG_LIGHTS_DUMP   = 858,
+
+    EVENT_SPAWN_CANCEL      = 860,
+    EVENT_SPAWN_ME          = 861,
+    EVENT_SPAWN_WHEELEDGRABBER = 862,
+    EVENT_SPAWN_WHEELEDSHOOTER = 863,
+    EVENT_SPAWN_PHAZERSHOOTER  = 864,
+    EVENT_SPAWN_BOTFACTORY  = 865,
+    EVENT_SPAWN_CONVERTER   = 866,
+    EVENT_SPAWN_DERRICK     = 867,
+    EVENT_SPAWN_POWERSTATION= 868,
+    EVENT_SPAWN_TITANIUM    = 869,
+    EVENT_SPAWN_TITANIUMORE = 870,
+    EVENT_SPAWN_URANIUMORE  = 871,
+    EVENT_SPAWN_POWERCELL   = 872,
+    EVENT_SPAWN_NUCLEARCELL = 873,
+
     EVENT_HYPER_PREV        = 900,
     EVENT_HYPER_NEXT        = 901,
     EVENT_HYPER_HOME        = 902,
