@@ -174,7 +174,7 @@ bool CGL21Device::Create()
 
     if (!InitializeGLEW())
     {
-        m_errorMessage = "An error occured while initializing GLEW.";
+        m_errorMessage = "An error occurred while initializing GLEW.";
         return false;
     }
 
@@ -242,7 +242,7 @@ bool CGL21Device::Create()
     glViewport(0, 0, m_config.size.x, m_config.size.y);
 
     // this is set in shader
-    int numLights = 8;
+    int numLights = 4;
 
     m_lights        = std::vector<Light>(numLights, Light());
     m_lightsEnabled = std::vector<bool> (numLights, false);
@@ -408,7 +408,7 @@ bool CGL21Device::Create()
         uni.fogColor = glGetUniformLocation(m_normalProgram, "uni_FogColor");
 
         uni.shadowColor = glGetUniformLocation(m_normalProgram, "uni_ShadowColor");
-        uni.lightingEnabled = glGetUniformLocation(m_normalProgram, "uni_LightingEnabled");
+        uni.lightCount = glGetUniformLocation(m_normalProgram, "uni_LightCount");
 
         uni.ambientColor = glGetUniformLocation(m_normalProgram, "uni_Material.ambient");
         uni.diffuseColor = glGetUniformLocation(m_normalProgram, "uni_Material.diffuse");
@@ -417,12 +417,6 @@ bool CGL21Device::Create()
         GLchar name[64];
         for (int i = 0; i < 8; i++)
         {
-            sprintf(name, "uni_Light[%d].Enabled", i);
-            uni.lights[i].enabled = glGetUniformLocation(m_normalProgram, name);
-
-            sprintf(name, "uni_Light[%d].Type", i);
-            uni.lights[i].type = glGetUniformLocation(m_normalProgram, name);
-
             sprintf(name, "uni_Light[%d].Position", i);
             uni.lights[i].position = glGetUniformLocation(m_normalProgram, name);
 
@@ -434,18 +428,6 @@ bool CGL21Device::Create()
 
             sprintf(name, "uni_Light[%d].Specular", i);
             uni.lights[i].specular = glGetUniformLocation(m_normalProgram, name);
-
-            sprintf(name, "uni_Light[%d].Attenuation", i);
-            uni.lights[i].attenuation = glGetUniformLocation(m_normalProgram, name);
-
-            sprintf(name, "uni_Light[%d].SpotDirection", i);
-            uni.lights[i].spotDirection = glGetUniformLocation(m_normalProgram, name);
-
-            sprintf(name, "uni_Light[%d].Exponent", i);
-            uni.lights[i].spotExponent = glGetUniformLocation(m_normalProgram, name);
-
-            sprintf(name, "uni_Light[%d].SpotCutoff", i);
-            uni.lights[i].spotCutoff = glGetUniformLocation(m_normalProgram, name);
         }
 
         // Set default uniform values
@@ -476,10 +458,7 @@ bool CGL21Device::Create()
 
         glUniform1f(uni.shadowColor, 0.5f);
 
-        glUniform1i(uni.lightingEnabled, 0);
-
-        for (int i = 0; i < 8; i++)
-            glUniform1i(uni.lights[i].enabled, 0);
+        glUniform1i(uni.lightCount, 0);
     }
 
     // Obtain uniform locations from interface rendering program and initialize them
@@ -594,6 +573,7 @@ void CGL21Device::ConfigChanged(const DeviceConfig& newConfig)
 
     // Reset state
     m_lighting = false;
+    m_updateLights = true;
 
     glViewport(0, 0, m_config.size.x, m_config.size.y);
 
@@ -721,36 +701,7 @@ void CGL21Device::SetLight(int index, const Light &light)
 
     m_lights[index] = light;
 
-    LightLocations &loc = m_uniforms[m_mode].lights[index];
-
-    glUniform4fv(loc.ambient, 1, light.ambient.Array());
-    glUniform4fv(loc.diffuse, 1, light.diffuse.Array());
-    glUniform4fv(loc.specular, 1, light.specular.Array());
-    glUniform3f(loc.attenuation, light.attenuation0, light.attenuation1, light.attenuation2);
-
-    if (light.type == LIGHT_DIRECTIONAL)
-    {
-        glUniform1i(loc.type, 1);
-        glUniform4f(loc.position, -light.direction.x, -light.direction.y, -light.direction.z, 0.0f);
-    }
-    else if (light.type == LIGHT_POINT)
-    {
-        glUniform1i(loc.type, 2);
-        glUniform4f(loc.position, light.position.x, light.position.y, light.position.z, 1.0f);
-
-        glUniform3f(loc.spotDirection, 0.0f, 1.0f, 0.0f);
-        glUniform1f(loc.spotCutoff, -1.0f);
-        glUniform1f(loc.spotExponent, 1.0f);
-    }
-    else if (light.type == LIGHT_SPOT)
-    {
-        glUniform1i(loc.type, 3);
-        glUniform4f(loc.position, light.position.x, light.position.y, light.position.z, 1.0f);
-
-        glUniform3f(loc.spotDirection, -light.direction.x, -light.direction.y, -light.direction.z);
-        glUniform1f(loc.spotCutoff, std::cos(light.spotAngle));
-        glUniform1f(loc.spotExponent, light.spotIntensity);
-    }
+    m_updateLights = true;
 }
 
 void CGL21Device::SetLightEnabled(int index, bool enabled)
@@ -760,7 +711,7 @@ void CGL21Device::SetLightEnabled(int index, bool enabled)
 
     m_lightsEnabled[index] = enabled;
 
-    glUniform1i(m_uniforms[m_mode].lights[index].enabled, enabled ? 1 : 0);
+    m_updateLights = true;
 }
 
 /** If image is invalid, returns invalid texture.
@@ -798,12 +749,13 @@ Texture CGL21Device::CreateTexture(ImageData *data, const TextureCreateParams &p
 
     result.originalSize = result.size;
 
-    glActiveTexture(GL_TEXTURE0);
-    glEnable(GL_TEXTURE_2D);
-
     glGenTextures(1, &result.id);
+
+    glActiveTexture(GL_TEXTURE0);
     glBindTexture(GL_TEXTURE_2D, result.id);
 
+    glEnable(GL_TEXTURE_2D);
+
     // Set texture parameters
     GLint minF = GL_NEAREST, magF = GL_NEAREST;
     int mipmapLevel = 1;
@@ -991,7 +943,7 @@ void CGL21Device::SetTexture(int index, const Texture &texture)
     glBindTexture(GL_TEXTURE_2D, texture.id);
 
     // Params need to be updated for the new bound texture
-    UpdateTextureStatus();
+    UpdateTextureState(index);
     UpdateTextureParams(index);
 }
 
@@ -1009,7 +961,7 @@ void CGL21Device::SetTexture(int index, unsigned int textureId)
     glBindTexture(GL_TEXTURE_2D, textureId);
 
     // Params need to be updated for the new bound texture
-    UpdateTextureStatus();
+    UpdateTextureState(index);
     UpdateTextureParams(index);
 }
 
@@ -1024,15 +976,54 @@ void CGL21Device::SetTextureEnabled(int index, bool enabled)
     if (same)
         return; // nothing to do
 
-    UpdateTextureStatus();
+    UpdateTextureState(index);
 }
 
-void CGL21Device::UpdateTextureStatus()
+void CGL21Device::UpdateTextureState(int index)
 {
-    for (int i = 0; i < 3; i++)
+    bool enabled = m_texturesEnabled[index] && (m_currentTextures[index].id != 0);
+    glUniform1i(m_uniforms[m_mode].textureEnabled[index], enabled ? 1 : 0);
+}
+
+void CGL21Device::UpdateLights()
+{
+    m_updateLights = false;
+
+    // If not in normal rendering mode, return immediately
+    if (m_mode != 0) return;
+
+    // Lighting enabled
+    if (m_lighting)
     {
-        bool enabled = m_texturesEnabled[i] && (m_currentTextures[i].id != 0);
-        glUniform1i(m_uniforms[m_mode].textureEnabled[i], enabled ? 1 : 0);
+        int index = 0;
+
+        // Iterate all lights
+        for (unsigned int i = 0; i < m_lights.size(); i++)
+        {
+            // If disabled, ignore and continue
+            if (!m_lightsEnabled[i]) continue;
+
+            // If not directional, ignore and continue
+            if (m_lights[i].type != LIGHT_DIRECTIONAL) continue;
+
+            Light &light = m_lights[i];
+            LightLocations &uni = m_uniforms[m_mode].lights[index];
+
+            glUniform4fv(uni.ambient, 1, light.ambient.Array());
+            glUniform4fv(uni.diffuse, 1, light.diffuse.Array());
+            glUniform4fv(uni.specular, 1, light.specular.Array());
+
+            glUniform4f(uni.position, -light.direction.x, -light.direction.y, -light.direction.z, 0.0f);
+
+            index++;
+        }
+
+        glUniform1i(m_uniforms[m_mode].lightCount, index);
+    }
+    // Lighting disabled
+    else
+    {
+        glUniform1i(m_uniforms[m_mode].lightCount, 0);
     }
 }
 
@@ -1044,6 +1035,12 @@ inline void CGL21Device::BindVBO(GLuint vbo)
     m_currentVBO = vbo;
 }
 
+inline void CGL21Device::BindTexture(int index, GLuint texture)
+{
+    glActiveTexture(GL_TEXTURE0 + index);
+    glBindTexture(GL_TEXTURE_2D, texture);
+}
+
 /**
   Sets the texture parameters for the given texture stage.
   If the given texture was not set (bound) yet, nothing happens.
@@ -1058,48 +1055,6 @@ void CGL21Device::SetTextureStageParams(int index, const TextureStageParams &par
     UpdateTextureParams(index);
 }
 
-void CGL21Device::SetTextureCoordGeneration(int index, TextureGenerationParams &params)
-{
-    /*
-    glActiveTexture(GL_TEXTURE0 + index);
-
-    for (int i = 0; i < 4; i++)
-    {
-        GLuint texCoordGen = TranslateTextureCoordinateGen(i);
-        GLuint texCoord = TranslateTextureCoordinate(i);
-
-        switch (params.coords[i].mode)
-        {
-        case TEX_GEN_NONE:
-            glDisable(texCoordGen);
-            break;
-        case TEX_GEN_OBJECT_LINEAR:
-            glEnable(texCoordGen);
-            glTexGeni(texCoord, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
-            glTexGenfv(texCoord, GL_OBJECT_PLANE, params.coords[i].plane);
-            break;
-        case TEX_GEN_EYE_LINEAR:
-            glEnable(texCoordGen);
-            glTexGeni(texCoord, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-            glTexGenfv(texCoord, GL_EYE_PLANE, params.coords[i].plane);
-            break;
-        case TEX_GEN_SPHERE_MAP:
-            glEnable(texCoordGen);
-            glTexGeni(texCoord, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
-            break;
-        case TEX_GEN_NORMAL_MAP:
-            glEnable(texCoordGen);
-            glTexGeni(texCoord, GL_TEXTURE_GEN_MODE, GL_NORMAL_MAP);
-            break;
-        case TEX_GEN_REFLECTION_MAP:
-            glEnable(texCoordGen);
-            glTexGeni(texCoord, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
-            break;
-        }
-    }
-    // */
-}
-
 void CGL21Device::UpdateTextureParams(int index)
 {
     assert(index >= 0 && index < static_cast<int>( m_currentTextures.size() ));
@@ -1163,6 +1118,8 @@ void CGL21Device::SetTextureStageWrap(int index, TexWrapMode wrapS, TexWrapMode
 void CGL21Device::DrawPrimitive(PrimitiveType type, const Vertex *vertices, int vertexCount,
                               Color color)
 {
+    if (m_updateLights) UpdateLights();
+
     BindVBO(0);
 
     Vertex* vs = const_cast<Vertex*>(vertices);
@@ -1190,6 +1147,8 @@ void CGL21Device::DrawPrimitive(PrimitiveType type, const Vertex *vertices, int
 void CGL21Device::DrawPrimitive(PrimitiveType type, const VertexTex2 *vertices, int vertexCount,
                               Color color)
 {
+    if (m_updateLights) UpdateLights();
+
     BindVBO(0);
 
     VertexTex2* vs = const_cast<VertexTex2*>(vertices);
@@ -1222,6 +1181,8 @@ void CGL21Device::DrawPrimitive(PrimitiveType type, const VertexTex2 *vertices,
 
 void CGL21Device::DrawPrimitive(PrimitiveType type, const VertexCol *vertices, int vertexCount)
 {
+    if (m_updateLights) UpdateLights();
+
     BindVBO(0);
 
     VertexCol* vs = const_cast<VertexCol*>(vertices);
@@ -1238,9 +1199,171 @@ void CGL21Device::DrawPrimitive(PrimitiveType type, const VertexCol *vertices, i
     glDisableClientState(GL_COLOR_ARRAY);
 }
 
+void CGL21Device::DrawPrimitive(PrimitiveType type, const void *vertices,
+    int size, const VertexFormat &format, int vertexCount)
+{
+    if (m_updateLights) UpdateLights();
+
+    BindVBO(0);
+
+    const char *ptr = reinterpret_cast<const char*>(vertices);
+
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glVertexPointer(format.vertex.size,
+        TransformType(format.vertex.type),
+        format.vertex.stride,
+        ptr + format.vertex.offset);
+
+    if (format.color.enabled)
+    {
+        glEnableClientState(GL_COLOR_ARRAY);
+        glColorPointer(format.color.size,
+            TransformType(format.color.type),
+            format.color.stride,
+            ptr + format.color.offset);
+    }
+    else
+        glColor4fv(format.color.values);
+
+    if (format.normal.enabled)
+    {
+        glEnableClientState(GL_NORMAL_ARRAY);
+        glNormalPointer(TransformType(format.normal.type),
+            format.normal.stride,
+            ptr + format.normal.offset);
+    }
+    else
+        glNormal3fv(format.normal.values);
+
+    glClientActiveTexture(GL_TEXTURE0);
+    if (format.tex1.enabled)
+    {
+        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+        glTexCoordPointer(format.tex1.size,
+            TransformType(format.tex1.type),
+            format.tex1.stride,
+            ptr + format.tex1.offset);
+    }
+    else
+        glTexCoord2fv(format.tex1.values);
+
+    glClientActiveTexture(GL_TEXTURE1);
+    if (format.tex2.enabled)
+    {
+        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+        glTexCoordPointer(format.tex2.size,
+            TransformType(format.tex2.type),
+            format.tex2.stride,
+            ptr + format.tex2.offset);
+    }
+    else
+        glTexCoord2fv(format.tex2.values);
+
+    glDrawArrays(TranslateGfxPrimitive(type), 0, vertexCount);
+
+    glDisableClientState(GL_VERTEX_ARRAY);
+
+    if (format.color.enabled) glDisableClientState(GL_COLOR_ARRAY);
+    if (format.normal.enabled) glDisableClientState(GL_NORMAL_ARRAY);
+
+    if (format.tex1.enabled)
+    {
+        glClientActiveTexture(GL_TEXTURE0);
+        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+    }
+
+    if (format.tex2.enabled)
+    {
+        glClientActiveTexture(GL_TEXTURE1);
+        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+    }
+}
+
+void CGL21Device::DrawPrimitives(PrimitiveType type, const void *vertices,
+    int size, const VertexFormat &format, int first[], int count[], int drawCount)
+{
+    if (m_updateLights) UpdateLights();
+
+    BindVBO(0);
+
+    const char *ptr = reinterpret_cast<const char*>(vertices);
+
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glVertexPointer(format.vertex.size,
+        TransformType(format.vertex.type),
+        format.vertex.stride,
+        ptr + format.vertex.offset);
+
+    if (format.color.enabled)
+    {
+        glEnableClientState(GL_COLOR_ARRAY);
+        glColorPointer(format.color.size,
+            TransformType(format.color.type),
+            format.color.stride,
+            ptr + format.color.offset);
+    }
+    else
+        glColor4fv(format.color.values);
+
+    if (format.normal.enabled)
+    {
+        glEnableClientState(GL_NORMAL_ARRAY);
+        glNormalPointer(TransformType(format.normal.type),
+            format.normal.stride,
+            ptr + format.normal.offset);
+    }
+    else
+        glNormal3fv(format.normal.values);
+
+    glClientActiveTexture(GL_TEXTURE0);
+    if (format.tex1.enabled)
+    {
+        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+        glTexCoordPointer(format.tex1.size,
+            TransformType(format.tex1.type),
+            format.tex1.stride,
+            ptr + format.tex1.offset);
+    }
+    else
+        glTexCoord2fv(format.tex1.values);
+
+    glClientActiveTexture(GL_TEXTURE1);
+    if (format.tex2.enabled)
+    {
+        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+        glTexCoordPointer(format.tex2.size,
+            TransformType(format.tex2.type),
+            format.tex2.stride,
+            ptr + format.tex2.offset);
+    }
+    else
+        glTexCoord2fv(format.tex2.values);
+
+    glMultiDrawArrays(TranslateGfxPrimitive(type), first, count, drawCount);
+
+    glDisableClientState(GL_VERTEX_ARRAY);
+
+    if (format.color.enabled) glDisableClientState(GL_COLOR_ARRAY);
+    if (format.normal.enabled) glDisableClientState(GL_NORMAL_ARRAY);
+
+    if (format.tex1.enabled)
+    {
+        glClientActiveTexture(GL_TEXTURE0);
+        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+    }
+
+    if (format.tex2.enabled)
+    {
+        glClientActiveTexture(GL_TEXTURE1);
+        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+    }
+}
+
 void CGL21Device::DrawPrimitives(PrimitiveType type, const Vertex *vertices,
     int first[], int count[], int drawCount, Color color)
 {
+    if (m_updateLights) UpdateLights();
+
     BindVBO(0);
 
     Vertex* vs = const_cast<Vertex*>(vertices);
@@ -1267,6 +1390,8 @@ void CGL21Device::DrawPrimitives(PrimitiveType type, const Vertex *vertices,
 void CGL21Device::DrawPrimitives(PrimitiveType type, const VertexTex2 *vertices,
     int first[], int count[], int drawCount, Color color)
 {
+    if (m_updateLights) UpdateLights();
+
     BindVBO(0);
 
     VertexTex2* vs = const_cast<VertexTex2*>(vertices);
@@ -1300,6 +1425,8 @@ void CGL21Device::DrawPrimitives(PrimitiveType type, const VertexTex2 *vertices,
 void CGL21Device::DrawPrimitives(PrimitiveType type, const VertexCol *vertices,
     int first[], int count[], int drawCount)
 {
+    if (m_updateLights) UpdateLights();
+
     BindVBO(0);
 
     VertexCol* vs = const_cast<VertexCol*>(vertices);
@@ -1460,6 +1587,8 @@ void CGL21Device::DrawStaticBuffer(unsigned int bufferId)
     if (it == m_vboObjects.end())
         return;
 
+    if (m_updateLights) UpdateLights();
+
     BindVBO((*it).second.bufferId);
 
     if ((*it).second.vertexType == VERTEX_TYPE_NORMAL)
@@ -1627,9 +1756,11 @@ void CGL21Device::SetRenderState(RenderState state, bool enabled)
     }
     else if (state == RENDER_STATE_LIGHTING)
     {
+        if (m_lighting == enabled) return;
+
         m_lighting = enabled;
 
-        glUniform1i(m_uniforms[m_mode].lightingEnabled, enabled ? 1 : 0);
+        m_updateLights = true;
 
         return;
     }
