@@ -510,6 +510,8 @@ protected:
 
     void        AutosaveRotate();
     void        Autosave();
+    void        QuickSave();
+    void        QuickLoad();
     bool        DestroySelectedObject();
     void        PushToSelectionHistory(CObject* obj);
     CObject*    PopFromSelectionHistory();
