@@ -121,7 +121,7 @@ Error CAutoLabo::StartAction(int param)
 {
     if ( m_phase != ALAP_WAIT )
     {
-        return ERR_UNKNOWN;
+        return ERR_OBJ_BUSY;
     }
 
     m_research = static_cast<ResearchType>(param);
