@@ -59,4 +59,11 @@ public:
     //! Damage the object, with the given force. Returns true if the object has been fully destroyed (assuming the object is destroyable, of course). If force == infinity, destroy immediately (this is the default value)
     /** NOTE: You should never assume that after this function exits, the object is destroyed, unless it returns true. Even if you specify force = infinity, if may still sometimes decide not to destroy the object. */
     virtual bool DamageObject(DamageType type, float force = std::numeric_limits<float>::infinity(), CObject* killer = nullptr) = 0;
+
+
+    //! Set the status that means the object is currently taking damage
+    virtual void        SetDamaging(bool damaging) = 0;
+    //! Is object currently taking damage?
+    virtual bool        IsDamaging() = 0;
+
 };
