@@ -148,7 +148,7 @@ CScreenSetup* CMainUserInterface::GetSetupScreen(Phase phase)
 
 void CMainUserInterface::ChangePhase(Phase phase)
 {
-    m_main->GetCamera()->SetType(Gfx::CAM_TYPE_DIALOG);
+    m_main->GetCamera()->SetType(Gfx::CAM_TYPE_NULL);
     m_engine->SetOverFront(false);
     m_engine->SetOverColor(Gfx::Color(0.0f, 0.0f, 0.0f, 0.0f), Gfx::ENG_RSTATE_TCOLOR_BLACK); // TODO: color ok?
 
@@ -213,9 +213,10 @@ void CMainUserInterface::ChangePhase(Phase phase)
 
     if ( IsMainMenuPhase(m_phase) )
     {
-        if (!m_sound->IsPlayingMusic() && m_sound->IsCachedMusic("music/Intro1.ogg"))
+        if (!m_sound->IsPlayingMusic())
         {
             m_sound->PlayMusic("music/Intro1.ogg", false);
+            m_sound->CacheMusic("music/Intro2.ogg");
         }
     }
 
